/* listing 4.1
 * Demo of the GtkTable widget.
 */

#include <gtk/gtk.h>

GtkWidget *frm_table;
GtkWidget *tbl;

/* The command button widgets below will be specified by their top left
 * location.  For example, the first one is row 1, column 1.
 * The second is in the row 2, column 1, etc.
 */

GtkWidget *cmd_1_1;
GtkWidget *cmd_2_1;
GtkWidget *cmd_4_1;
GtkWidget *cmd_1_2;
GtkWidget *cmd_1_3;
GtkWidget *cmd_3_2;
GtkWidget *cmd_2_3;
GtkWidget *cmd_3_3;
GtkWidget *chk_1;

void destroy_main();
void make_table_buttons();
void attach_widgets();
void cmd_1_1_clicked();
void cmd_2_1_clicked();
void cmd_4_1_clicked();
void cmd_1_2_clicked();
void cmd_1_3_clicked();
void cmd_3_2_clicked();
void cmd_2_3_clicked();
void cmd_3_3_clicked();

gint main(gint argc, gchar *argv[])
{

   gtk_init(&argc, &argv);

   frm_table = gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_window_set_title(GTK_WINDOW(frm_table), "Table Demo");

   gtk_signal_connect(GTK_OBJECT(frm_table),
        "destroy",
        GTK_SIGNAL_FUNC(destroy_main),
        NULL);

   make_table_buttons();

   /* Initial setup of command buttons using defaults. 
    *
    * The only reason we use gtk_table_attach_defaults() here
    * instead of attach_widgets() is to demonstrate the 
    * gtk_table_attach_defaults() function.
    */

   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_1_1, 0, 1, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_2_1, 0, 1, 1, 3);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_4_1, 0, 4, 3, 4);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_1_2, 1, 2, 0, 2);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_1_3, 2, 3, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_3_2, 1, 2, 2, 3);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_3_3, 2, 3, 2, 3);
   gtk_table_attach_defaults(GTK_TABLE(tbl), cmd_2_3, 2, 4, 1, 2);

   chk_1 = gtk_check_button_new_with_label("Check Btn");

   gtk_table_attach_defaults(GTK_TABLE(tbl), chk_1, 3, 4, 0, 1);

   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 

   gtk_widget_show_all (frm_table);
   gtk_main ();
   return 0;

}

void destroy_main()
{   gtk_main_quit();
}

void attach_widgets(gint _OPTIONS, gint _PACKING)
{
   gtk_table_attach(GTK_TABLE(tbl), cmd_1_1, 0, 1, 0, 1, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_2_1, 0, 1, 1, 3, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_4_1, 0, 4, 3, 4, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_1_2, 1, 2, 0, 2, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_1_3, 2, 3, 0, 1, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_3_2, 1, 2, 2, 3, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_3_3, 2, 3, 2, 3, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), cmd_2_3, 2, 4, 1, 2, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
   gtk_table_attach(GTK_TABLE(tbl), chk_1, 3, 4, 0, 1, _OPTIONS, _OPTIONS, _PACKING, _PACKING);
}


void make_table_buttons()
{
   tbl = gtk_table_new(4, 4, TRUE);

   /* The parameters for the previous call are:  rows, columns, homogenous. 
    */

   cmd_1_1 = gtk_button_new_with_label("Set Padding 9");
   gtk_signal_connect(GTK_OBJECT(cmd_1_1),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_1_1_clicked),
        NULL);
   cmd_2_1 = gtk_button_new_with_label("Set\nOptions\nTo\nFILL");
   gtk_signal_connect(GTK_OBJECT(cmd_2_1),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_2_1_clicked),
        NULL);
   cmd_4_1 = gtk_button_new_with_label("Set Options To SHRINK");
   gtk_signal_connect(GTK_OBJECT(cmd_4_1),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_4_1_clicked),
        NULL);
   cmd_1_2 = gtk_button_new_with_label("Set\nOptions\nTo\nEXPAND");
   gtk_signal_connect(GTK_OBJECT(cmd_1_2),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_1_2_clicked),
        NULL);
   cmd_1_3 = gtk_button_new_with_label("Set\nOptions\nTo\nFILL\nand\nSHRINK");
   gtk_signal_connect(GTK_OBJECT(cmd_1_3),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_1_3_clicked),
        NULL);
   cmd_3_2 = gtk_button_new_with_label("Set\nOptions\nTo\nSHRINK\nand\nEXPAND");
   gtk_signal_connect(GTK_OBJECT(cmd_3_2),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_3_2_clicked),
        NULL);
   cmd_3_3 = gtk_button_new_with_label("Set\nOptions\nTo\nFILL\nand\nEXPAND");
   gtk_signal_connect(GTK_OBJECT(cmd_3_3),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_3_3_clicked),
        NULL);
   cmd_2_3 = gtk_button_new_with_label("Set Options To\nFILL and EXPAND and SHRINK");
   gtk_signal_connect(GTK_OBJECT(cmd_2_3),
        "clicked",
        GTK_SIGNAL_FUNC(cmd_2_3_clicked),
        NULL);
   chk_1 = gtk_check_button_new_with_label("Check Btn");
}

void cmd_1_1_clicked()
{
   /* This is something new that we haven't talked about yet.  As will
    * be the case with all the following command button callbacks, we
    * will actually be destroying and then recreating the table of 
    * command buttons that the user presses.  This is because there
    * is no *_set_* command for the table widget.
    * 
    * TODO:  verify that statement!!
    *        also, destroy no need for g_free??
    *
    * Even if there was, we could still do it this way if we wanted
    * to.  Granted, this gets a little tedious, but if we change the
    * user interface, we know all the places to make changes in order
    * to maintain our layout.
    */

   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(0, 9);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_2_1_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_FILL, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_4_1_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_SHRINK, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_1_2_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_EXPAND, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_1_3_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_FILL | GTK_SHRINK, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_3_2_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_SHRINK | GTK_EXPAND, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_3_3_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_FILL | GTK_EXPAND, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}

void cmd_2_3_clicked()
{
   gtk_widget_destroy(tbl);
   make_table_buttons();
   attach_widgets(GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0);
   gtk_container_add(GTK_CONTAINER(frm_table), tbl); 
   gtk_widget_show_all (frm_table);
}
