/* listing 4.3
 * Demo of the GtkFixed Widget.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
/* Needed for the random number function in the callback
 * at the bottom of the listing.
 */

GtkWidget *frm_fixed;
GtkWidget *fixed_main;
GtkWidget *cmd1;

void destroy_main();
void cmd1_clicked();

gint main(gint argc, gchar *argv[])
{
   gtk_init(&argc, &argv);

   frm_fixed = gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_window_set_title(GTK_WINDOW(frm_fixed), "Fixed Widget Demo");

   gtk_signal_connect(GTK_OBJECT(frm_fixed),
        "destroy",
        GTK_SIGNAL_FUNC(destroy_main),
        NULL);

   fixed_main = gtk_fixed_new();

   /* The GtkFixed widget is very simple to use.  As shown
    * below, you give it the coordinates at which to place
    * a child widget.  The widget attributes of fill, expand,
    * padding, etc., are irrelevent here;  the child widgets
    * placed on to the fixed widget will default to the 
    * minimum required to show correctly.  This is the same
    * behavior of the GtkFixed widget;  it will size itself
    * to the minimum required to show all of it's child
    * widgets.  If a child widget is moved to an area outside
    * of the GtkFixed's current size, the fixed widget will
    * grow just enough to properly show the widget.  If, 
    * however, the child widgets are moved to a location
    * that would allow the window to shrink, the fixed
    * widget will not automatically shrink to the minimum
    * required size.
    *
    * This can lead to some unexpected behavior.  For example,
    * let's say the initial position is 1,1.  Then, the
    * position changes to 100, 100.  If the position then
    * changes to 50, 50, the window size will not change.
    * If, however, the position chages to 150, 50, the
    * window will change size - to be just enough to
    * properly show the widget.  Another way to put it is
    * that the window will be redrawn whenever the child
    * widgets push an outside boundary of the window, either
    * pushing the right or bottom boundaries out.  Then,
    * window will re-size.  Thus, you could go from an 
    * essentially square window to a long and thin one, 
    * and vice-versa.
    *
    * While the GtkFixed widget is simple to implement,
    * you will probably want to use the GtkLayout widget
    * in most instances.  Examine it before deciding 
    * which will best suit your project.
    */ 

   cmd1 = gtk_button_new_with_label("Button 1");
   gtk_signal_connect(GTK_OBJECT(cmd1),
        "clicked",
        GTK_SIGNAL_FUNC(cmd1_clicked),
        NULL);
   gtk_fixed_put(GTK_FIXED(fixed_main), cmd1, 1, 100);

   /* The 3rd and 4th paramaters of the previous call
    * are distance from left edge of window in pixels,
    * and distance from top edget of window in pixels, 
    * respectively.
    */

   gtk_container_add(GTK_CONTAINER(frm_fixed), fixed_main); 

   gtk_widget_show_all (frm_fixed);
   gtk_main ();
   return 0;

}

void destroy_main()
{   gtk_main_quit();
}

void cmd1_clicked()
{
   gint x_pos, y_pos;

   x_pos = rand();
   y_pos = rand();

   /* The poor man's bounded random number generator.  In 
    * this case, the target is for a number between 1 and 200.
    */

   do{
      x_pos = x_pos - 200;
   } while (x_pos > 200);

   do{
      y_pos = y_pos - 200;
   } while (y_pos > 200);

   gtk_fixed_move(GTK_FIXED(fixed_main), GTK_WIDGET(cmd1), x_pos, y_pos);
}

