/* listing 4.4
 * Demo of the GtkLayout Widget.
 */

#include <gtk/gtk.h>
#include <stdlib.h>

GtkWidget *frm_layout;
GtkWidget *layout_main;
GtkWidget *cmd1;

void destroy_main();
void cmd1_clicked();

gint main(gint argc, gchar *argv[])
{
   gtk_init(&argc, &argv);

   frm_layout = gtk_window_new(GTK_WINDOW_TOPLEVEL);

   gtk_window_set_title(GTK_WINDOW(frm_layout), "Layout Widget Demo");

   gtk_signal_connect(GTK_OBJECT(frm_layout),
        "destroy",
        GTK_SIGNAL_FUNC(destroy_main),
        NULL);

   layout_main = gtk_layout_new(NULL, NULL); 

   /* Other than the call above to gtk_layout_new(),
    * which requires a vertical and horzontal adjustment
    * object as parameters, you could take the previous
    * listing and change every instance of "fixed" to
    * "layout" and the program should work.
    *
    * However, there are also added lines in this
    * program to demonstrate the Layout widget
    * functionality.
    */

   cmd1 = gtk_button_new_with_label("Button 1");
   gtk_signal_connect(GTK_OBJECT(cmd1),
        "clicked",
        GTK_SIGNAL_FUNC(cmd1_clicked),
        NULL);
   gtk_layout_put(GTK_LAYOUT(layout_main), cmd1, 1, 100);

   gtk_container_add(GTK_CONTAINER(frm_layout), layout_main); 

   gtk_widget_show_all (frm_layout);
   gtk_main ();
   return 0;

}

void destroy_main()
{   gtk_main_quit();
}

void cmd1_clicked()
{
   gint x_pos, y_pos;

   x_pos = rand();
   y_pos = rand();

   /* The poor man's bounded random number generator.  In 
    * this case, the target is for a number between 1 and 200.
    */

   do{
      x_pos = x_pos - 200;
   } while (x_pos > 200);

   do{
      y_pos = y_pos - 200;
   } while (y_pos > 200);

   /* Here is one big difference between the Fixed and Layout widgets.
    * It is best to freeze and thaw the Layout widget, especially if
    * you have a lot of child widgets to move around and show simultaneously.
    * This avoids redrawing the screen unnecessarily.
    */

   gtk_layout_freeze(GTK_LAYOUT(layout_main));

   gtk_layout_move(GTK_LAYOUT(layout_main), GTK_WIDGET(cmd1), x_pos, y_pos);
 
   gtk_layout_thaw(GTK_LAYOUT(layout_main));
}

