/* The following line is needed for compilation
 * on Windows - comment the line out for Linux gcc.
 */

#include        <windows.h>


#include <gtk/gtk.h>
#include <mysql.h>

/* string.h is needed for the strlen() function. */
/* time.h is needed for the date/time functions. */
/* stdio.h is needed for the write to file operations. */

#include <string.h>
#include <time.h>
#include <stdio.h>

#include "support.h"
#include "ddc.h"

#define SERVER "einstein"
#define LOGIN  "com_user"
#define PASS   "syL0U812"
#define DB     "commish"

GtkWidget *frm_login;
GtkWidget *frm_table_display;
GtkWidget *frm_commissions;

/* The following are some generic variables that are used in
 * many of the functions below.  There is no reason that they
 * HAVE to be global;  if they aren't global, however, they 
 * would need to be re-declared in each function and passed
 * in and out of functions repeatedly.
 */

MYSQL       *conx;
MYSQL_RES   *result_set;
MYSQL_ROW   row;
gchar       *sql;
gchar       *user_login;

/* Function prototypes. */

/*
void log_event(gchar *event_msg);
void set_message(gchar *msg);
gboolean check_login_and_password();
gboolean set_new_password();
GList *get_table_names();
void fill_table_combos();
void archive_tbl_commissions();
void load_tbl_revenue();
void process_customer_commissions();
void process_worldwide();
void process_state();
void process_area(gchar *area);
*/

/* Begin function code. */

void log_event(gchar *event_msg)
{
    /* This routine will quietly log who attempts to log in,
     * and other potential security flags.
     *
     * The one thing that would be important is the machine name
     * of the attempting login.  However, since that has 
     * been taken care of with a GRANT statement for each
     * machine that can potentially log in, it won't be
     * included here.
     *
     * The g_get_user_name() function gets the login
     * of the user at the operating system level.
     * In this case, the application maintains a separate list
     * of logins and passwords;  the g_get_user_name() function
     * is used as a double check on the attempted login
     * and may be useful if someone who shouldn't have access
     * is attempting to log in.  Obviously, it could
     * also be used to "pass through" the machine login
     * to the application, avoiding the need for a separate
     * login and password per applicaiton.
     */

   sql = g_strconcat("insert into tbl_logged_events ",
                     "(user_login, event) ",
                     "values ('", user_login, "', '", event_msg,
                     " local login: ", g_get_user_name(), "')",
                     0L
                    );

   // g_print ("log_event init of mysql...\n");
   conx = mysql_init(0L);
   if (conx == 0L) g_print("init problem...\n");

   // g_print("real connecting...\n");
   conx = mysql_real_connect(conx, SERVER, LOGIN,
                             PASS, DB, 0, 0L, 0);
   // g_print("real connect done...\n");
   if (conx == 0L) g_print("real connect problem...\n");

   // g_print("querying...\n");
   mysql_query (conx, sql);

   // g_print("closing...\n");
   mysql_close (conx);


}

void set_message(gchar *msg) 
{
  /* There are 3 places in the application to communicate with the user:
   * the label widget at the bottom of frm_login, and the statusbar
   * widgets at the bottom of each of the other two forms.  To make
   * it easy, just push the message to all three.
   */

  g_print("set_message: %s\n", msg);

  gtk_label_set_text(GTK_LABEL(lookup_widget(frm_login, 
                     "lbl_messages")), msg);

  gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_table_display,
                                   "statusbar")), 1, msg);

  gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_commissions,
                                   "statusbar_processing")), 1, msg);

  /* Of course, this could also send msg to log_event for 
   * really detailed logging.
   */
  
  g_print("Exiting set_message...\n");
};

gboolean check_login_and_password()
{
    g_print("inside check_login_and_password.\n");

    user_login = gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_login, "entry_name")),
                             0, -1);

    g_print("user_login is %s\n", user_login);

    sql = g_strconcat("select * from ",
                      "tbl_security where user_login = '", 
                      user_login,
                      "' and pass_word = password('",
                      gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_login, "entry_password")),
                             0, -1),
                      "')", 0L);

   /* This appliction is going to take a bit of a different tack
    * with the database connection mechanism.  It will connect
    * and close the connection for each database operation, 
    * rather than attempt to maintain an open connection.
    * 
    * Since the code next is the first attempt to connect,
    * the error messages will be returned to the user if
    * something goes wrong.  For later connections in this
    * application, it will be assumed that if the application
    * gets past these initial checks, the connection is
    * available and reliable.
    */

   g_print("attemtpting init...\n");
   if ((conx = mysql_init(0L)) == 0L)
      {   
         set_message("Unable to initialize mysql.");
         return FALSE;
      }
   g_print("init done...\n");
 
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   if (conx == 0L)
      {   
         mysql_close(conx);
         set_message("Unable to initialize database connection.");
         return FALSE;
      }
   g_print("real_connect done...\n");
 
   if (mysql_query (conx, sql) != 0)
      {   
         g_print("query failed...\n");
         mysql_close(conx);
         set_message("Unable to query database.");
         return FALSE;
      }

   set_message("Connected to commissions database...");

   result_set = mysql_store_result(conx);
   if (mysql_num_rows (result_set) == 1)
      {
          mysql_free_result (result_set);
          mysql_close(conx);
          set_message("Logged in");
          log_event(g_strconcat("Login of ", 
                                user_login, 0L));
          return TRUE;
      }
   else
      { 
          mysql_free_result (result_set);
          mysql_close(conx);
          set_message("Unable to log in.  See Administrator.");
         
          log_event(g_strconcat("Failed Login attempt by ", 
                                user_login, 0L));
          return FALSE;
      }

}

gboolean set_new_password()
{
   gchar     *str_new_pwd;
   gchar     *str_new_again;

   /* First, some basic checking.  Make sure that the
    * new passwords have some length.  Are the two
    * new passwords the same?  etc...  It is normal that logins are
    * not case sensitive, but that passwords are.
    */

   g_print("starting set_new_password...\n");
   log_event(g_strconcat(user_login, " attempting to set "
                         "new password. ", 0L));

   str_new_pwd =  gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_login, "entry_new_password")),
                             0, -1);
   str_new_again =  gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_login, 
                             "entry_new_password_again")),
                             0, -1);

   if (strlen(str_new_pwd) == 0)
     {
        set_message("New password(s) not of sufficient length.");
        return FALSE;
     }

   if (strcmp(str_new_pwd, str_new_again) == 0)
      {
         set_message("New passwords matched...\n");
         sql = g_strconcat("update tbl_security ",
                           "set pass_word = password('", str_new_pwd, "') ",
                           "where user_login = '", user_login, "'",
                           0L);

         g_print("sql is %s\n", sql);
  
         conx = mysql_init(0L);
         conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);
         if (mysql_query (conx, sql) != 0)
            {
               set_message("Unable to change password.  "
                           "Database query failure.\n"
                           "Press Login to connect with "
                           "current name and password.");
               mysql_close(conx);
               return FALSE;
            }
         else
            {
               set_message("New password set.");
               mysql_close(conx);

               log_event(g_strconcat(user_login, " changed password. ", 0L));
               return TRUE;

            }

      }
   else
      {
         set_message("New password(s) not matched.\n");
         return FALSE;
      }

   return FALSE;

}

GList *get_table_names()
{
    GList *list_of_tables = 0L;

    conx = mysql_init(0L);
    conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);
    
    sql = g_strconcat("select * from tbl_permissions ",
                      "where login = '", user_login, "'",
                      0L);

    if (mysql_query (conx, sql) != 0)
        {
            set_message("Unable to retrieve list of tables.");
            log_event(g_strconcat("Unable to retrieve list of tables",
                             " for user ", user_login, 0L));
        }
    else
        {
            result_set = mysql_store_result (conx);
            while (( row = mysql_fetch_row (result_set)) != 0L)
                { 
                  if (strcmp(row[1], "process") != 0) 
                     list_of_tables = g_list_append(list_of_tables, row[1]);
                  else 
                     gtk_widget_show(lookup_widget(frm_table_display,
                                "cmd_process"));
                }
        }

    mysql_close(conx);
    return list_of_tables;

}

void fill_table_combos()
{
    /* This function fills the two combo boxes that list
     * the tables the user has access to.
     */

    gtk_combo_set_popdown_strings(GTK_COMBO(lookup_widget(frm_table_display,
                     "cbo_tables")), get_table_names());

    gtk_combo_set_popdown_strings(GTK_COMBO(lookup_widget(frm_commissions,
                     "cbo_table")), get_table_names());
}

void archive_tbl_commissions()
{
   gchar       *str_table_name;
   time_t      now;
   gchar       *rows_before, *rows_after;

   /* This function will archive the existing tbl_commissions
    * and prepare it for a new round of processing.
    *
    * It will proceed along the following steps:
    *     1.  first, get a count of the number of records currently
    *         in the table.
    *     2.  Create a new table with a make table query that
    *         transfers the contents of tbl_commissions to the
    *         new table.  This new table will be the date and
    *         time of it's creation, for archival purposes.
    *     3.  Get a count of records in the newly-created table
    *         and compare to the original table.
    *     4.  If the table record counts match, delete all rows
    *         from tbl_commissions and indicate success to the
    *         user.
    */

   log_event("Archiving tbl_commissions. ");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   sql = "select count(*) from tbl_commissions";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            set_message("Query Failure in record count on tbl_commissions.");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                               "lbl_archive")), "Query Failure in record "
                               "count on tbl_commissions.\n  Contact "
                               "administrator.");
            
            /* Nothing to do but abort. */
            mysql_close(conx);
            return;
        }
   else
       {
            /* Query Success... */
            result_set = mysql_store_result (conx);
            row = mysql_fetch_row (result_set);
            g_print("tbl_commissions row count is %s...\n", row[0]);
         
            rows_before = row[0];
 
       }
   /* Now to create the new table. */

   time(&now);
   str_table_name = g_strstrip((gchar *) ctime(&now));
   str_table_name = g_strdelimit(str_table_name, " :,", '_');

   g_print("new table name is %s\n", str_table_name);

   /* The command used below is the CREATE TABLE command.  While
    * there are various ways to use this command, in this case the
    * syntax that will give the desired result here is:
    *
    *    create table tbl select-statement
    *
    * where tbl is the new table name and select-statement is a 
    * valid sql select statement.  This creates a new table from
    * the results of a valid select statement, rather than specifying
    * the columns explicitly.
    */

   sql = g_strconcat("create table ", str_table_name, 
                     " select * from tbl_commissions", 0L);

   if (mysql_query (conx, sql) != 0)
      {
            /* Query failure...  */
            set_message(g_strconcat("Query Failure in create table on ", 
                      str_table_name, 0L));
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                               "lbl_archive")), "Query Failure in create "
                               "table statement.\n  Note the date and time "
                               "and contact administrator.");
            
            /* Nothing to do but abort. */
            mysql_close(conx);
            return;
      }
   else
      {
            /* Query Success... */
      }

   /* Now to count the rows in the new table and make sure it
    * compares to the number from the original.
    */

   sql = g_strconcat("select count(*) from ", str_table_name, 0L);

   if (mysql_query (conx, sql) != 0)
      {
            /* Query failure...  */
            set_message(g_strconcat("Query Failure: count of table ", 
                      str_table_name, 0L));
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                               "lbl_archive")), "Query Failure in count "
                               "of new table.\n  Note the date and time "
                               "and contact administrator.");
            
            mysql_close(conx);
            return;
      }
   else
      {
            /* Query Success... */

      }

   result_set = mysql_store_result (conx);
   row = mysql_fetch_row (result_set);
   g_print("new table row count is %s...\n", row[0]);
  
   rows_after = row[0];

   /* Compare the answers of the two count queries.
    * Whether integer or string, the answers should
    * be identical.  In this case, because of the way
    * mysql queries are returned, the results are 
    * strings.
    */

   if (g_strcasecmp(rows_after, rows_before) != 0)
      {
          /* Difference in query results;  something
           * has gone wrong...
           */
         
          set_message("Failure comparing archived table "
                      "to current table.");
          gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_archive")), "Query Failure in comparison "
                      "of archived and new tables.\n  Note the date and time "
                      "and contact administrator.");

          mysql_close(conx);
          return;
      }
   else
      {
         /* The number of rows compares correctly, that 
          * is, they have the same number of rows.
          * At this point, all indications are that
          * tbl_commissions has been correctly archived
          * and that the program can continue with
          * the processing operation.
          */
      }
    
   sql = "Delete from tbl_commissions";
   if (mysql_query (conx, sql) != 0)
      {
          /* Query failure...  */
          set_message("Failure deleting from tbl_commissions");
          gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_archive")), "Query Failure in delete "
                      "of records from tbl_commissions.\n  "
                      "Note the date and time "
                      "and contact administrator.");

          mysql_close(conx);
          return;
      }
   else
      {
          /* Succeeded in executing the query.
           * Remember, the mysql_query() function
           * returns whethere or not it was given a 
           * properly formed SQL statement that could
           * be executed, not whether or not the
           * query actually deleted anything (in this
           * case).  Therefore, the software needs to 
           * check the count of rows in the table.
           *
           * Continue processing...
           */ 
      }

   sql = "select count(*) from tbl_commissions";
   if (mysql_query (conx, sql) != 0)
      {
         /* Query failure...  */
         set_message("Failure getting count from tbl_commissions");
         gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_archive")), "Query Failure in count "
                      "of records from tbl_commissions.\n  "
                      "Note the date and time "
                      "and contact administrator.");

         mysql_close(conx);
         return;
      }
   else
      {
         /* Query executed ok... */
      }

   result_set = mysql_store_result (conx);
   row = mysql_fetch_row (result_set);
   g_print("old table row count is %s...\n", row[0]);

   if (g_strcasecmp(row[0], "0") != 0)
      {
        /* Difference in query results;  something
         * has gone wrong...
         */

        set_message("Not 0 rows in tbl_commissions");
        gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_archive")), "Query Failure: not 0 "
                      "rows in tbl_commissions.\n  "
                      "Note the date and time "
                      "and contact administrator.");

        mysql_close(conx);
        return;
      }
   else
      {
         /* All has gone correctly. */ 
         mysql_close(conx);
         gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                               "lbl_archive")), "Archive succeeded."
                           );
         log_event("Successfully archived tbl_commissions. ");
      }

}

void load_tbl_revenue()
{
   /* This function will load the text file into tbl_revenue.
    * It does the same thing as Listing 8.1 without the 'drop
    * table' and 'select count(*)' statements.  In fact, the LOAD
    * DATA statement is identical except for the escape characters
    * before the double-quotes in the first line.
    */

   log_event("Improrting to tbl_revenue. ");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting import...\n");

   /* First, empty tbl_revenue and check that it is empty before
    * importing the text file.
    */
   sql = "delete from tbl_revenue";

   if (mysql_query (conx, sql) != 0)
      {
         /* Query failure...  */
         mysql_close(conx);
         set_message("Query failure: delete from tbl_revenue");
         gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                   "lbl_import")), "Query Failure deleteing from "
                   "tbl_revenue.\n  "
                   "Note the date and time "
                   "and contact administrator.");
         return;
      }
   else
      {
         g_print("Query success, delete from tbl_revenue.\n");
      }

   /* Check to see that there are, in fact, 0 rows in tbl_revenue. */

   sql = "select count(*) from tbl_revenue";

   if (mysql_query (conx, sql) != 0)
      {
         /* Query failure...  */
         mysql_close(conx);
         set_message("Query failure: count of rows from tbl_revenue");
         gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                   "lbl_import")), "Query Failure counting rows from "
                   "tbl_revenue.\n  "
                   "Note the date and time "
                   "and contact administrator.");
         return;
      }
   else
      {
         /* Query success. */
      }

   result_set = mysql_store_result (conx);
   row = mysql_fetch_row (result_set);
   g_print("tbl_revenue row count is %s...\n", row[0]);

   if (g_strcasecmp(row[0], "0") != 0)
      {
        /* Not 0 rows in tbl_revenue.  Importing now would
         * result in incorrect data processing, so exit.
         */

        set_message("Not ZERO rows in tbl_revenue");
        gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_import")), "Query Failure: not ZERO "
                      "rows in tbl_revenue.\n  "
                      "Note the date and time "
                      "and contact administrator.");

        mysql_close(conx);
        return;
      }
   else
      {
        /* Query ok, so continue... */
      }

   sql = "LOAD DATA INFILE "
         " \"/mnt/DOS_hda2/newriders/book/ch8/tbl_revenue.txt\" "
         "INTO TABLE tbl_revenue "
         "FIELDS TERMINATED BY \',\' "
         "IGNORE 1 LINES "
         "(customer_number,"
         " customer_name,"
         " salesperson,"
         " country,"
         " city,"
         " state,"
         " zip,"
         " ship_date,"
         " item_number,"
         " order_quantity,"
         " unit_price,"
         " net_value,"
         " invoice_number,"
         " invoice_date,"
         " invoice_quantity,"
         " invoice_value)";

   g_print("sql is: %s\n", sql);

   /* Note that in order for this application to execute the query currently
    * in sql, the following permission must be set.  In the mysql 
    * database (which is actually a database within the mysql server), 
    * in table 'user', the 'File_priv' must be set to Y for the 
    * specified user on the specified machine.  In this
    * case, the user is com_user, and the machine name will be whatever 
    * machine the user "processor" is on.
    *
    * Don't forget the 'flush privileges' command from the mysql> prompt
    * to reload the privileges to the database (or reboot the machine, which
    * also works).
    */

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            g_print("Error number is %i...\n", mysql_errno(conx) );
            g_print("Error description is %s...\n", mysql_error(conx) );

            mysql_close(conx);
            set_message("Query failure in LOAD DATA statement");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_import")), "Query Failure in "
                      "LOAD DATA statement.\n  "
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
           /* query success */ 
        }

   sql = "select count(*) from tbl_revenue";
   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure in select count from tbl_revenue");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_import")), "Query Failure in "
                      "select count from tbl_reveue..\n  "
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            /* Query succeeded...  */
        }

   result_set = mysql_store_result (conx);
   row = mysql_fetch_row (result_set);
   g_print("tbl_revenue row count is %s...\n", row[0]);

   if (g_strcasecmp(row[0], "0") != 0)
      {
          /* Correct, tbl_revenue should be populated now, so
           * report success back to the user.
           */
           gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_import")), g_strconcat("Import succeeded. ", 
                      row[0], " rows imported.", 0L));
 
      }
   else 
      {
          /* There are 0 rows in the table after the import operation,
           * which is incorrect; the table should be populated now.
           */
           gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_import")), "0 rows in tbl_revenue after "
                      "import operation...\n  "
                      "Note the date and time "
                      "and contact administrator.");

      }

    mysql_close(conx);

}

void process_customer_commissions()
{

   log_event("Processing commissions on salepeople in tbl_revenue. ");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting customer processing...\n");

   sql = "insert into tbl_commissions "
         "(salesperson, invoice_num, commission) "
         "select tbl_revenue.salesperson, "
         "tbl_revenue.invoice_number, "
         "tbl_people.commission * tbl_revenue.invoice_value "
         "from tbl_revenue, tbl_people "
         "where tbl_revenue.salesperson = tbl_people.salesperson";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure in process_customer_commissions.");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_customers")), "Query Failure "
                      "procssing by saleperson in tbl_revenue. \n  "
                      "Could not execute Insert query. \n"
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                   "lbl_customer")), g_strconcat("Customer Succeeded. ",
                   g_strdup_printf("%d",(gint) mysql_affected_rows(conx)), 
                   " rows inserted.", 0L));

            mysql_close(conx);
            set_message("process_customer_commissions succeeded.");
        }

}

void process_worldwide()
{
   /* In this step, the software should process commissions for those 
    * salespeople who are paid a commission based on every invoice
    * issued worldwide.  This will be a simple operation as there are 
    * only 2 people in tbl_people_paid_on_worldwide.
    */

   log_event("Processing worldwide commissions.");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting worldwide processing...\n");

   sql = "insert into tbl_commissions "
         "(salesperson, invoice_num, commission) "
         "select tbl_people_paid_on_worldwide.salesperson, "
         "       tbl_revenue.invoice_number, "
         "       tbl_people.commission * tbl_revenue.invoice_value "
         "from tbl_people_paid_on_worldwide, tbl_revenue, tbl_people "
         "where tbl_people.salesperson = "
         "      tbl_people_paid_on_worldwide.salesperson ";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure in process_worldwide.");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_worldwide")), "Query Failure "
                      "procssing worldwide. \n  "
                      "Could not execute Insert query. \n"
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                  "lbl_worldwide")), g_strconcat("Worldwide Succeeded. ",
                  g_strdup_printf("%d",(gint) mysql_affected_rows(conx)), 
                  " rows inserted.", 0L));
            
            mysql_close(conx);
            set_message("process_worldwide succeeded.");
        }

}

void process_area(gchar *area)
{
   /* This function will allow processing of the country and state
    * steps.  This is due to the similar names of the label
    * widgets and MySQL tables.  Unfortunately, because cmd_zip and
    * lbl_zip are not named cmd_zipcode and lbl_zipcode, this proceedure
    * will not work for the zipcode processing also.  C'est la guerre.
    */

   log_event(g_strconcat("Processing ", area, " commissions.", 0L));

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting %s processing...\n", area);

   sql = g_strconcat(
         "insert into tbl_commissions (salesperson, invoice_num, commission) "
         "select tbl_people_paid_on_", area, ".salesperson, "
         "       tbl_revenue.invoice_number, "
         "       tbl_people.commission * tbl_revenue.invoice_value "
         "from tbl_people_paid_on_", area, ", tbl_revenue, tbl_people "
         "where tbl_people.salesperson=tbl_people_paid_on_", 
                area, ".salesperson"
         "  and tbl_people_paid_on_", area, ".", area, "=tbl_revenue.", area, 
         0L);

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message(g_strconcat("Query failure in process_", 
                                    area, ".", 0L));
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      g_strconcat("lbl_", area, 0L))), 
                      g_strconcat( "Query Failure "
                      "procssing ", area, ". \n  "
                      "Could not execute Insert query. \n"
                      "Note the date and time "
                      "and contact administrator.", 0L));
            return;
        }
   else
        {
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      g_strconcat("lbl_", area, 0L))), 
                      g_strconcat(area, " Succeeded. ",
                      g_strdup_printf("%d",(gint) mysql_affected_rows(conx)), 
                      " rows inserted.", 0L));
            
            mysql_close(conx);
            set_message(g_strconcat("process ", area, " succeeded.", 0L));
        }

}

void process_zipcode()
{
   log_event("Processing zipcodes.");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting zipcode processing...\n");

   sql = "insert into tbl_commissions (salesperson, invoice_num, commission) "
         "select tbl_people_paid_on_zipcode.salesperson, "
         "       tbl_revenue.invoice_number, "
         "       tbl_people.commission * tbl_revenue.invoice_value "
         "from tbl_people_paid_on_zipcode, tbl_revenue, tbl_people "
         "where tbl_people.salesperson=tbl_people_paid_on_zipcode.salesperson"
         "  and tbl_people_paid_on_zipcode.zipcode=tbl_revenue.zip";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure in process_zipcode");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_zip")), "Query Failure "
                      "procssing zipcodes. \n  "
                      "Could not execute Insert query. \n"
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_zip")), g_strconcat("Zipcodes Succeeded. ",
                      g_strdup_printf("%d",(gint) mysql_affected_rows(conx)), 
                      " rows inserted.", 0L));
            
            mysql_close(conx);
            set_message("process zipcodes succeeded.");
        }

}

void process_paid_on_people()
{
   log_event("Processing people paid on other people.");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting paid-on-others processing...\n");

   sql = "insert into tbl_commissions (salesperson, invoice_num, commission) "
         "select tbl_people_paid_on_other_people.person_paid, "
         "tbl_revenue.invoice_number, "
         "tbl_people.commission * tbl_revenue.invoice_value "
         "from tbl_revenue, tbl_people, tbl_people_paid_on_other_people "
         "where tbl_revenue.salesperson = "
         "      tbl_people_paid_on_other_people.paid_on "
         "  and tbl_people.salesperson = "
         "      tbl_people_paid_on_other_people.person_paid";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure in process_paid_on_people");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_people")), "Query Failure "
                      "procssing those paid on others. \n  "
                      "Could not execute Insert query. \n"
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_people")), g_strconcat("People paid on other",
                      " People Succeeded. ",
                      g_strdup_printf("%d",(gint) mysql_affected_rows(conx)), 
                      " rows inserted.", 0L));
            
            mysql_close(conx);
            set_message("process paid-on-others succeeded.");
        }

}

void write_salespeople_statements()
{
   gchar *file_name;
   FILE  *fp;
   gchar *current_salesperson = "nobody";

   log_event("Writing salespeople statements.");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting write of salespeople statments.\n");
   gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_salespeople")), "Retrieving records "
                      "from the database...\n "
                      "This may take a few minutes..."
                      );

   gtk_widget_show(lookup_widget(frm_commissions, "lbl_salespeople")); 

   sql = "select tbl_commissions.salesperson, "
         "       tbl_commissions.invoice_num, "
         "       tbl_commissions.commission, "
         "       tbl_people.first_name, "
         "       tbl_people.last_name, "
         "       tbl_revenue.customer_name, "
         "       tbl_revenue.item_number, "
         "       tbl_revenue.order_quantity "
         "from tbl_commissions, tbl_people, tbl_revenue "
         "where tbl_commissions.salesperson = tbl_people.salesperson "
         "  and tbl_commissions.invoice_num = tbl_revenue.invoice_number "
         "order by tbl_commissions.salesperson, tbl_commissions.invoice_num";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure: writing salespeople statements");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_salespeople")), "Query Failure "
                      "writing salespeople statements. \n  "
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            result_set = mysql_store_result (conx);

            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_salespeople")), g_strconcat("Writing salespeople ",
                      "statements to disk...\n ",
                      "Total Records: ", 
                      g_strdup_printf("%d",(gint) mysql_num_rows(result_set)), 
                      0L));
            
            set_message("Retrieved records for salesman statements.");

            /* The query executed ok and result_set has returned
             * the total number of rows, so it is ok to continue...
             */
        }

   /* Now to move on to writing to the file.  Since each salesperson 
    * will have their own file and result_set is organized by 
    * salesperson, the software will iterate through the rows 
    * and create a new file each time the value for salesperson
    * changes.
    *
    * To start, need a file name.
    *
    * Fetch the first row to get things started. 
    */

   if ((row = mysql_fetch_row (result_set)) == 0L)
      {  
         /* Unable to get first row. */
         set_message("Unable to retrieve first row from result_set.");
    
         return;
      }
   else
     {
        /* Ok to continue */
     }

   /* In order to avoid a core dump when the fclose() operation
    * is encountered, set up for the first salesperson.
    */

   current_salesperson = row[0];
   file_name = g_strconcat(current_salesperson, "-",
                     row[4], ".xls", 0L);

   g_print("Current file is %s.\n", file_name);

   /* Open the new file. */

   if ((fp = fopen (file_name, "w")) == 0L)
        {
              set_message(g_strconcat("Unable to open first file ", 
                          file_name, 
                          " for write operation.", 0L));
              return;
        }
   else
        {
              /* Write the neader information. */
              fprintf(fp, "Commissions detail file for ");
              fprintf(fp, current_salesperson);
              fprintf(fp, ", ");
              fprintf(fp, row[3]);  /* The person's first name. */
              fprintf(fp, "\n");

              fprintf(fp, "All currency amounts are in US Dollars.\n\n");

              fprintf(fp, "Invoice Number\t");
              fprintf(fp, "Commission Amt\t");
              fprintf(fp, "Customer Name\t");
              fprintf(fp, "Item Number\t");
              fprintf(fp, "Order Quantity\n");
        }

   /* The file will be the salesperson number plus the
    * person's last name.  Since the salespeople are use
    * to getting files that work in Excel, this function
    * will write a tab-delimited file.
    */

   do {
         if (g_strcasecmp(row[0], current_salesperson) == 0)
           {
              /* Still on the same salesperson. */

              fprintf(fp, row[1]); fprintf(fp, "\t");
              fprintf(fp, row[2]); fprintf(fp, "\t");
              fprintf(fp, row[5]); fprintf(fp, "\t");
              fprintf(fp, row[6]); fprintf(fp, "\t");
              fprintf(fp, row[7]); fprintf(fp, "\n");

           }
         else
           {
              /* row now contains a new salesperson.
               * Close the file currently open.
               */

              fclose(fp);

              /* Create a new file name. */

              current_salesperson = row[0];
              file_name = g_strconcat(current_salesperson, "-",
                                row[4], ".xls", 0L);

              g_print("Current file is %s.\n", file_name);

              /* Open the new file. */

              if ((fp = fopen (file_name, "w")) == 0L)
                 {
                    /* Unable to open the file for write operation
                     * for some reason.
                     */
         
                     set_message(g_strconcat("Unable to open file ", 
                                 file_name, 
                                 " for write operation.", 0L));

                     return;
                 }
              else
                 {
                     /* File opened and ready for write operation. */

                 }

              /* Write the neader information. */
              fprintf(fp, "Commissions detail file for ");
              fprintf(fp, current_salesperson);
              fprintf(fp, " - ");
              fprintf(fp, row[4]);  /* The person's last name. */
              fprintf(fp, ", ");
              fprintf(fp, row[3]);  /* The person's first name. */
              fprintf(fp, "\n");

              fprintf(fp, "All currency amounts are in US Dollars.\n\n");

              fprintf(fp, "Invoice Number\t");
              fprintf(fp, "Commission Amt\t");
              fprintf(fp, "Customer Name\t");
              fprintf(fp, "Item Number\t");
              fprintf(fp, "Order Quantity\n");

           }

      } while ((row = mysql_fetch_row (result_set)) != 0L);

   g_print("Exiting write_salesperson_statements.\n");
   mysql_close(conx);

}

void write_country_manager_statements()
{
   gchar *file_name;
   FILE  *fp;
   gchar *current_country = "none";

   log_event("Writing country manager statements.");

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("Starting write of country manager statments.\n");
   gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_country_managers")), "Retrieving records "
                      "from the database...\n "
                      "This may take a few minutes..."
                      );

   sql = "select tbl_people.country, tbl_commissions.salesperson, "
         "       tbl_commissions.commission, "
         "       tbl_commissions.commission * tbl_exchange_rates.rate "
         "from tbl_commissions, tbl_people, tbl_exchange_rates "
         "where tbl_commissions.salesperson = tbl_people.salesperson "
         "  and tbl_people.country = tbl_exchange_rates.country "
         "group by tbl_commissions.salesperson "
         "order by tbl_people.country ";

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            mysql_close(conx);
            set_message("Query failure: "
                        "writing country manager's statements");
            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_country_managers")), "Query Failure "
                      "writing country manager's statements. \n  "
                      "Note the date and time "
                      "and contact administrator.");
            return;
        }
   else
        {
            result_set = mysql_store_result (conx);

            gtk_label_set_text(GTK_LABEL(lookup_widget(frm_commissions, 
                      "lbl_country_managers")), 
                      g_strconcat("Writing country ",
                      "manager's statements to disk...\n ",
                      "Total Records: ", 
                      g_strdup_printf("%d",(gint) mysql_num_rows(result_set)), 
                      0L));
            
            set_message("Retrieved records for country "
                        "manager's statements.");

            /* The query executed ok and result_set has returned
             * the total number of rows, so it is ok to continue...
             */
        }

   /* Fetch the first row to get things started. */

   if ((row = mysql_fetch_row (result_set)) == 0L)
      {  
         /* Unable to get first row. */
         set_message("Unable to retrieve first row from result_set "
                     "for country manager's statements.");
    
         return;
      }
   else
     {
        /* Ok to continue */
     }

   current_country = row[0];
   file_name = g_strconcat(current_country, ".xls", 0L);

   g_print("Current file is %s.\n", file_name);

   /* Open the new file. */

   if ((fp = fopen (file_name, "w")) == 0L)
        {
              set_message(g_strconcat("Unable to open first ",
                          "country file: ", 
                          file_name, 
                          ", for write operation.", 0L));
              return;
        }
   else
        {
              /* Write the neader information. */
              fprintf(fp, "Commissions detail file for country: ");
              fprintf(fp, current_country);
              fprintf(fp, "\n");

              fprintf(fp, "Salesperson\t");
              fprintf(fp, "US Dollars\t");
              fprintf(fp, "Local Currency\n");
        }

   do {

         if (g_strcasecmp(row[0], current_country) == 0)
           {
              /* Still on the same country. */

              fprintf(fp, row[1]); fprintf(fp, "\t");
              fprintf(fp, row[2]); fprintf(fp, "\t");
              fprintf(fp, row[3]); fprintf(fp, "\n");

           }
         else
           {
              /* row now contains a new country.
               * Close the file currently open.
               */

              fclose(fp);

              /* Create a new file name. */

              current_country = row[0];
              file_name = g_strconcat(current_country, ".xls", 0L);

              g_print("Current file is %s.\n", file_name);

              /* Open the new file. */

              if ((fp = fopen (file_name, "w")) == 0L)
                 {
                    /* Unable to open the file for write operation
                     * for some reason.
                     */
         
                     set_message(g_strconcat("Unable to open country file ", 
                                 file_name, 
                                 " for write operation.", 0L));

                     return;
                 }
              else
                 {
                     /* File opened and ready for write operation. */
                 }

              /* Write the neader information. */
              fprintf(fp, "Commissions detail file for country: ");
              fprintf(fp, current_country);
              fprintf(fp, "\n");

              fprintf(fp, "Salesperson\t");
              fprintf(fp, "US Dollars\t");
              fprintf(fp, "Local Currency\n");

           }

      } while ((row = mysql_fetch_row (result_set)) != 0L);

   g_print("Exiting write_country_manager_statements.\n");
   mysql_close(conx);

}

void insert_row()
{
   gchar *str_table_name;

   str_table_name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(lookup_widget(
                     frm_commissions, "cbo_table"))->entry));

   sql = g_strconcat("insert into ", str_table_name,
                      "() values()", 0L);


   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   g_print("sql is %s\n", sql);

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            g_print("sql not ok in insert_row().\n");
            mysql_close(conx);
            g_print("connection closed.\n");
            set_message(g_strconcat("Query failure: inserting to table",
                         str_table_name, 0L));
        }
    else
        {
            g_print("sql formatted ok.\n");
            set_message(g_strconcat("New row number is: ", 
                    g_strdup_printf("%d",(gint) mysql_insert_id(conx)),
                    " in table ", str_table_name,
                    0L));

        }

   log_event(g_strconcat("insert query on ", str_table_name, 0L));
}

void update_row()
{
   gchar *str_table_name;

   str_table_name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(lookup_widget(
                     frm_commissions, "cbo_table"))->entry));

   /* The following sql statement is built assuming that it
    * will modify a character column, not numeric.  Note,
    * however, that MySQL will do explicit type conversion
    * from character to numeric in this case, and update numeric columns
    * if the conversion succeeds.  However, if the user 
    * attempts to incorrectly update a numeric column type 
    * to a character value, the value being updated will
    * be set to 0.
    */

   sql = g_strconcat("update ", str_table_name, " set ",
                gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_commissions, 
                             "entry_column_name")), 0, -1), 
                " = '",
                gtk_editable_get_chars(GTK_EDITABLE(
                             lookup_widget(frm_commissions, 
                             "entry_new_value")), 0, -1),
                "'  where line_number = ", 
                g_strdup_printf("%d", 
                gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
                        lookup_widget(frm_commissions, 
                        "spinbutton_update_line")))),
                0L);

   g_print("sql is: %s\n", sql);

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            g_print("sql not ok in update_row().\n");
            mysql_close(conx);
            g_print("connection closed.\n");
            set_message(g_strconcat("Query failure: updating row in ",
                         str_table_name, 0L));
        }
    else
        {
            g_print("sql formatted ok.\n");
            set_message(g_strconcat("Updated row in table ", 
                    str_table_name, 0L));

        }

    mysql_close (conx);

    log_event(g_strconcat(user_login, " update query: ", 
                          g_strdelimit(g_strstrip(sql), "*'\"", '^'), 
                          0L));

}

void delete_row()
{
   gchar *str_table_name;

   str_table_name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(lookup_widget(
                     frm_commissions, "cbo_table"))->entry));

   sql = g_strconcat("delete from ", str_table_name, " where ",
                "line_number = ", 
                g_strdup_printf("%d", 
                gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
                        lookup_widget(frm_commissions, 
                        "spinbutton_delete_row")))),
                0L);
   g_print("sql is: %s\n", sql);

   conx = mysql_init(0L);
   conx = mysql_real_connect(conx, SERVER, LOGIN, 
                             PASS, DB, 0, 0L, 0);

   if (mysql_query (conx, sql) != 0)
        {
            /* Query failure...  */
            g_print("sql not ok in delete_row().\n");
            mysql_close(conx);
            g_print("connection closed.\n");
            set_message(g_strconcat("Query failure: deleting row from ",
                         str_table_name, 0L));
        }
    else
        {
            g_print("sql formatted ok.\n");
            set_message(g_strconcat("Deleted row  from table ", 
                    str_table_name, 0L));

        }

    log_event(g_strconcat(user_login, " delete query: ", 
                          g_strdelimit(g_strstrip(sql), "*'\"", '^'), 
                          0L));

}
