#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

/* unistd.h is needed for the execl() command in 
 * on_cmd_open_click().
 */

#include <unistd.h>

/* stdlib.h is needed for the pid_t type (process
 * identifier) in on_cmd_open_click(). 
 */

#include <stdlib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "kbi_utils.h"


void
on_cmd_close_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
   g_print("on_cmd_close_clicked\n");
   gtk_main_quit();
}


void
on_cmd_open_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
   pid_t pid;
   gchar *exe_name = "./kbi";
   gchar *report_name = "another kbi";

   g_print("on_cmd_open_clicked\n");

   /* Since the executable exists, get it's name and the
    * report name for the execl() call below.
    */

   report_name = get_report_name();
   g_print("report_name is %s\n", report_name);

   if (g_strcasecmp(report_name, "") == 0)
     {
        g_print("No report selected.");
        return;
     }

   exe_name = get_exe_name(report_name);

   g_print("exe_name is %s\n", exe_name);

   if (g_strcasecmp(exe_name, "") == 0)
     {
        g_print("No executable returned.\n");
        return;
     }

   exe_name = g_strconcat("/usr/local/bin/", exe_name, 0L);

   /* Check that the target executable is on this machine
    * in the place that it should be:  /usr/local/bin/.
    */

//   exe_name = "/mnt/DOS_hda2/newriders/book/ch09/src/ww";

   g_print("Check that file exists.\n");

   if (check_file_exists(exe_name) == FALSE)
      {
         /* The target executable doesn't exist... */ 
         g_print("Target exe not found.\n");

         return;
      }

//   report_name = "wwcomms";

   /* This section requires a bit of an explanation.  The fork() call
    * creates a copy of the running program in a separate process
    * space.  That is, it - the "clild" process - has it's own
    * copy of the program, including variables, memory, etc.
    *  
    * Following that is the execl() call inside the if statemtent.
    * This changes the existing program and replaces it with the
    * program called in the first parameter. 
    */

   g_print("Fork off a new process.\n");
   pid = fork();

   if (pid == 0)
     {
        /* If we have entered this if statement, then this must
         * be the child process that was fork()-ed off in the
         * previous statements.  Therefore, this (child) process
         * should change itself to become the "target" process,
         * that is, the program that needs to be run.
         */

        g_print("initiating child process.\n");
        execl (exe_name, report_name);
        g_print("child process launched.\n");
     }
   /* else parent, so continue... */
     
}


gboolean
on_frm_kbi_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
   g_print("on_frm_kbi_delete_event\n");
   gtk_main_quit();
   return FALSE;
}


void
on_frm_kbi_show                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
   g_print("on_frm_kbi_show\n");

   if (authenticate_user())
     {
         fill_lst_reports();
     }

}


void
on_frm_kbi_realize                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
   g_print("on_frm_kbi_realize\n");
}


