#include <windows.h>

#include <gtk/gtk.h>
#include <mysql.h>

/* stdlib.h is needed for the atoi/atof call. */

#include <stdlib.h>

#include "support.h"

#define	SERVER  "einstein"
#define	LOGIN   "kbi_user"
#define PASS    "dengissa"
#define DB      "kbi"

#define BASELINE_CHART    350
#define BASELINE_MONTHS   370
#define SCALE_DENOMINATOR 1235
#define BAR_WIDTH         25

GtkWidget *frm_barline;

MYSQL           *conx;
MYSQL_RES       *result_set;
MYSQL_ROW       row;
gchar           *sql;

gchar *month_names[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                          "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

gint month_bar_amounts[12];
gint month_line_amounts[12];
gint month_indents[12];

void draw_bar(gint bar_left, gint bar_width, gint bar_height, gchar *bar_color);

GdkGC *get_gc(gchar *in_color)
{
   GdkGC    *gc;
   GdkColor my_color;

   gc = gdk_gc_new(GTK_WIDGET(lookup_widget(frm_barline,
           "drawingarea1"))->window);

   if (!gdk_color_parse(in_color, &my_color))
      {
         g_print("Unable to parse %s.\n", in_color);
         gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
               "statusbar1")), 1, "Unable to parse color.\n");
         return 0L;
      }

   if (!gdk_colormap_alloc_color(gdk_colormap_get_system(),
        &my_color, FALSE, TRUE))
      {
         g_print("Unable to allocate colormap for %s.\n", in_color);
         gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
               "statusbar1")), 1, "Unable to allocate colormap.\n");
         return 0L;
      }

   gdk_gc_set_foreground(gc, &my_color);

   return gc;

}

void draw_axis_and_scales()
{
   GdkGC    *gc;
   gint     i;
   gint     from_left = 90;

   gc = get_gc("Black");

   gdk_draw_string(
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
       gc, 5, 100, "Sales (Bar)");

   gdk_draw_string(
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
       gc, 5, 130, "Quota (Line)");

   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           gc, from_left -1, 10, from_left -1, BASELINE_MONTHS - 20);

   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           gc, from_left, BASELINE_MONTHS - 20, 550, BASELINE_MONTHS - 20);


   /* Establish the spacing for the months, both the names along
    * the horizontal axis and the bars also.
    */

   for (i=0; i<=11; i++)
     {
         month_indents[i] = from_left;

         gdk_draw_string(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
           gc, from_left, BASELINE_MONTHS, month_names[i]);

         from_left += 40;

     }

}

void draw_bar_chart()
{
   gint i;

   /* The first thing necessary here is to figure out scale.  In
    * this case, the largest value in the data is 400,000.  In the
    * "drawingarea", the left line is drawn between vertical pixel
    * 10 at the top and 350 at the bottom, leaving a vertical distance
    * of 340 pixels in which to represent values from 0 to
    * 400,000.
    *
    * So, to calculate the scale for that area, take the maximum value
    * plus a bit of padding, for this exercise the value will be 420,000.
    * Divide that by 340 gives a value of (approximately) 1235.  So,
    * if you then turn the calculation around and divide the 400,000
    * value by 1235, you should never get more than 340.  Thus, the
    * software can take all the values given to it in the 300,000 to
    * 400,000 range and divide them by 1235 in order to draw the
    * bars to scale correctly.
    *
    * Note that the software could of course query the data
    * with an SQL max() statment to determine what this program
    * sets with a #define statment.  In other words, the 1235 value
    * used here could be calculated from the data itself.
    *
    * It is from this discussion that MAX_VALUE is defined in the
    * header section of this file.
    */

   /* Make repeat calls to draw_bar() for each month. */

   for (i=0;  i<=11; i++)
      {
         draw_bar(month_indents[i], BAR_WIDTH,
                  month_bar_amounts[i]/SCALE_DENOMINATOR, "Red");

      }
}

void draw_bar(gint bar_left, gint bar_width, gint bar_height, gchar *bar_color)
{
   gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc(bar_color), TRUE, bar_left,
             BASELINE_CHART - bar_height,
             bar_width, bar_height);

}

void draw_line(gint from_left_1, gint from_top_1,
               gint from_left_2, gint from_top_2)

{
   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           get_gc("Blue"), from_left_1, from_top_1, from_left_2, from_top_2);


}


void draw_line_chart()
{
   gint i;
   gint from_left, from_top;

   /* The first thing necessary is to calculate the starting point
    * for the first line.  In this case, that is the middle of the
    * "January" bar for the measurement from the left of the window
    * and the vertical will be month_line_amounts[0].
    */

   from_left = month_indents[0] + BAR_WIDTH/2;
   from_top  = BASELINE_CHART - month_line_amounts[0]/SCALE_DENOMINATOR;

   for (i=1;  i<=11; i++)
      {
         /*
          */

         draw_line(from_left, from_top,
                   month_indents[i] + BAR_WIDTH/2,
                   BASELINE_CHART - month_line_amounts[i]/SCALE_DENOMINATOR);

         gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc("RoyalBlue"),
             TRUE, from_left - 2, from_top - 2, 3, 3);

         from_left = month_indents[i] + BAR_WIDTH/2;
         from_top  = BASELINE_CHART - month_line_amounts[i]/SCALE_DENOMINATOR;

      }

   /* Need to plot a point at the December quota data point as
    * the finishing touch.
    */

   gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc("RoyalBlue"),
             TRUE, from_left - 2, from_top - 2, 3, 3);

}

gboolean load_data()
{
   conx = mysql_init(0L);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1,"mysql_init problem");
        return FALSE;
     }
   /* else continue on... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_init ok");

   conx = mysql_real_connect(conx, SERVER, LOGIN, PASS, DB, 0, 0L, 0);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1,"mysql_real_connect problem");
        return FALSE;
     }
   /* else continue on... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_real_connect ok");

   sql = g_strconcat("select * from tbl_sales_vs_quota ",
                     "  order by sales_month", 0L);

   g_print("sql is %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1,"mysql_query problem");
        g_print("mysql_query problem in load_data.\n");
        mysql_close(conx);
        return FALSE;
      }
   /* else continue... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_query ok");

   result_set = mysql_store_result(conx);

   /* At this point, result_set holds a list of the reports that
    * are available to be viewed.  Create a GList structure to
    * hold that list.
    */

   while (( row = mysql_fetch_row (result_set)) != 0)
     {
        static gint i = 0;

        month_bar_amounts[i] = atoi(row[2]);
        month_line_amounts[i] = atoi(row[3]);

        g_print("data is %d and %d\n", month_bar_amounts[i],
                                        month_line_amounts[i]);

        i++;
     }

   mysql_close(conx);

   return TRUE;

}
