#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <mysql.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "ddc.h"

GtkWidget *frm_tabular;

void
on_frm_tabular_show                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  gchar     *sql;
  MYSQL     *conx;
  GtkWidget *scrolledwindow1;
  GtkWidget *statusbar1;

  /* When frm_tabular opens, it needs to show the top salespeople
   * in descending order in terms of commissions paid.  In order
   * for it to do that, this routine must do the following things
   * in this order:
   *
   * In the "commish" database:
   * 1.  Drop table tbl_top_salespeople if it exists.
   * 2.  Create tbl_top_salespeople by drawing data from tbl_commissions.
   * 3.  Select from the recently-created tbl_top_salespeople.
   *
   * Of course, the drop and create steps could be added to
   * the ww application of Chapters 8, 9, and 10, as a final
   * step, reducing the need for it here.
   */

  g_print("on_frm_tabular_show.\n");

  /* First, connect to the database. */

  conx = mysql_init(0L);
  conx = mysql_real_connect(conx, "einstein", "com_user",
                            "syL0U812", "commish", 0, 0L, 0);

  if (conx == 0L) 
      {
          g_print("Unable to connect to database.\n");
          gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_tabular,
                      "statusbar1")), 1, "Unable to Connect to database.");
          return;
      }

  g_print("connected to commish db.\n");

  sql = "drop table if exists tbl_top_salespeople";

  if (mysql_query (conx, sql) != 0)
    {
       g_print("Error dropping table.\n");
       g_print("Error number is %i\n", mysql_errno (conx));
       g_print("Error msg is %s\n", mysql_error (conx));
       gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_tabular,
                      "statusbar1")), 1, "Error dropping table.");
       mysql_close (conx);
       return;

    }

  g_print("table dropped.\n");

  sql = g_strconcat("create table tbl_top_salespeople ",
                    "select distinct salesperson, ",
                           "sum(commission) as commission, "
                           "count(*) as Count_rows "
                    "from tbl_commissions "
                    "group by salesperson", 0L);

  if (mysql_query (conx, sql) != 0)
    {
       g_print("Error creating table.\n");
       g_print("Error number is %i\n", mysql_errno (conx));
       g_print("Error msg is %s\n", mysql_error (conx));
       gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_tabular,
                      "statusbar1")), 1, "Error creating table.");
       mysql_close (conx);
       return;

    }

  g_print("table created.\n");

  mysql_close (conx);

  g_print("First connection closed.\n");

  sql = "tbl_top_salespeople order by commission DESC";

  gtk_widget_destroy(GTK_WIDGET(lookup_widget(frm_tabular,
                "scrolledwindow1")));

  g_print("Calling create_ddc.\n");
  scrolledwindow1 = create_ddc(sql, "einstein", "com_user", 
                               "syL0U812", "commish", frm_tabular);
  g_print("Returned from create_ddc.\n");
 
  gtk_widget_ref(scrolledwindow1);
  gtk_object_set_data_full(GTK_OBJECT(frm_tabular),
               "scrolledwindow1", scrolledwindow1,
               0L);

  gtk_box_pack_start(GTK_BOX(lookup_widget(frm_tabular,
               "vbox1")), scrolledwindow1, TRUE, TRUE, 0);

  gtk_widget_show(scrolledwindow1);

  /* Unfortunately, the packing box widgets don't have any
   * way to insert a child widget at a certain position;  only
   * start and end.  Therefore, destroy the statusbar widget
   * created in Glade and create a new one.  
   *
   * Remember, however, that the statusbar was needed prior
   * to this point to communicate with user.
   */
  gtk_widget_destroy(GTK_WIDGET(lookup_widget(frm_tabular,
          "statusbar1")));

  statusbar1 = gtk_statusbar_new();

  gtk_box_pack_start(GTK_BOX(lookup_widget(frm_tabular,
               "vbox1")), statusbar1, FALSE, FALSE, 0);
  gtk_widget_show(statusbar1);

  gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_tabular,
                      "statusbar1")), 1, "Done.");
}


gboolean
on_frm_tabular_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  g_print("on_frm_delete_event.\n");
  
  gtk_main_quit();
  return FALSE;
}

