#include <gtk/gtk.h>
#include <mysql.h>

/* time.h is needed for the calls to find the current
 * month and year.
 */

#include <time.h>

/* stdlib.h is needed for the atoi/atof call. */

#include <stdlib.h>

#include "support.h"

#define	SERVER  "einstein"
#define	LOGIN   "kbi_user"
#define PASS    "dengissa"
#define DB      "kbi"

GtkWidget *frm_piechart;

MYSQL           *conx;
MYSQL_RES       *result_set;
MYSQL_ROW       row;
gchar           *sql;

gint            number_of_divisions = 0;
gint            total_sales = 0;

/* This assumes there will never be more than 10 divisions. */

gchar           *division_names[10];
gint            division_sales[10];
GdkColor        color[10];
gchar           *color_choices[10] = {"Yellow", "Blue", "Red", "Gray", "Black",
                        "White", "Green", "Purple", "Orange", "Gold"};

void draw_legend(gint posit_x, gint posit_y,
                 gchar* legend_text, GdkColor my_color);

void draw_pie_wedge(gint start_deg, gint end_deg, GdkColor my_color)
{
   GdkGC  *gc;

   gc = gdk_gc_new(GTK_WIDGET(lookup_widget(frm_piechart,
                   "drawingarea1"))->window);

   if (!gdk_colormap_alloc_color(gdk_colormap_get_system(),
          &my_color, FALSE, TRUE))
     {
      g_print("unable to allocate colormap\n");
      gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
          "statusbar1")), 1, "Unable to allocate colormap.");
     }

   gdk_gc_set_foreground(gc, &my_color);

   //g_print("Drawing Arc.\n");
   gdk_draw_arc( GTK_WIDGET(lookup_widget(frm_piechart,
          "drawingarea1"))->window,
          gc, TRUE, 10, 10, 350, 350, start_deg*64, end_deg*64);

   //g_print("exiting draw_pie_wedge\n");
   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
          "statusbar1")), 1, "Done with draw_pie_wedge");
}

void parse_colors()
{
   gint     i;

   for (i=0; i<10; i++)
       {

        if (!gdk_color_parse(color_choices[i], &color[i]))
          {
            g_print("unable to parse color %d.\n", i);
            gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                 "statusbar1")), 1, "Unable to parse colors.");
          }
       }
}

void draw_pie_chart()
{
   gint   i;
   gint   current_pct;
   gint   current_degree = 0;
   gint   legend_vertical = 405;

   //g_print("Number of divisions is %d\n", number_of_divisions);

   for (i=1; i<=number_of_divisions; i++)
     {

        /* First calculate what percentage the current pie
         * slice is compared to the total.
         */

        //g_print("div sales i - 1 is %d\n", division_sales[i-1]);
        //g_print("total sales is %d\n", total_sales);

        /* This is truncating, so need to compensate..? */

        current_pct = ((gdouble) division_sales[i - 1] /
                       (gdouble) total_sales) * 100 + .5;

        //g_print("current percentage is %d\n", current_pct);

        /* If this is the final pie wedge, might as well
         * close the circle rather than leave a sliver open
         * that may be caused by rounding.
         */

        /* The second number is degress of arc, not ending degrees. */

        if (i == number_of_divisions)
            {
              /* This is the final pie wedge;  might as well
               * go ahead and close the circle, rather than
               * showing a small slice caused by rounding
               * errors.
               */
              draw_pie_wedge(current_degree,
                     360 - current_degree, color[i - 1]);
            }
        else
            {
              draw_pie_wedge(current_degree,
                     current_pct * 3.6, color[i - 1]);
            }

        current_degree += current_pct * 3.6;

        draw_legend(legend_vertical, 0,
                    g_strconcat(division_names[i - 1], " - ",
                    g_strdup_printf("%d", division_sales[i - 1]),
                    " (", g_strdup_printf("%d", current_pct),
                    "%)", 0L),
                    color[i - 1]);

        legend_vertical += 20;

        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                 "statusbar1")), 1, "Done drawing pie chart.");
     }
}

void draw_legend(gint posit_x, gint posit_y,
                 gchar* legend_text, GdkColor my_color)
{

   GdkGC  *gc;
   GdkColor black;

   gc = gdk_gc_new(GTK_WIDGET(lookup_widget(frm_piechart,
                   "drawingarea1"))->window);

   if (!gdk_colormap_alloc_color(gdk_colormap_get_system(),
          &my_color, FALSE, TRUE))
       {
            gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                 "statusbar1")), 1, "Unable to allocate colormap.");
       }

   gdk_gc_set_foreground(gc, &my_color);

   gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_piechart,
        "drawingarea1"))->window, gc, TRUE, 20, posit_x, 10, 10);

   if (!gdk_color_parse("Black", &black))
        {
            g_print("unable to parse black.\n");
            gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                 "statusbar1")), 1, "Unable to parse Black.");
        }

   if (!gdk_colormap_alloc_color(gdk_colormap_get_system(),
          &black, FALSE, TRUE))
       {
          g_print("unable to allocate colormap\n");
          gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                 "statusbar1")), 1, "Unable to allocate colormap.");
       }

   gdk_gc_set_foreground(gc, &black);

   gdk_draw_string(
        GTK_WIDGET(lookup_widget(frm_piechart, "drawingarea1"))->window,
        GTK_WIDGET(lookup_widget(frm_piechart, "drawingarea1"))->style->font,
        gc, 35, posit_x + 10, legend_text);

}

gboolean load_data()
{

   gchar *str_year, *str_month;
   struct tm *ptr;
   time_t now;

   conx = mysql_init(0L);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1,"mysql_init problem");
        return FALSE;
     }
   /* else continue on... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1, "mysql_init ok");

   conx = mysql_real_connect(conx, SERVER, LOGIN, PASS, DB, 0, 0L, 0);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1,"mysql_real_connect problem");
        return FALSE;
     }
   /* else continue on... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1, "mysql_real_connect ok");

   /* Figure out the current month and year, then extract the
    * previous month's data.
    */

   time(&now);
   ptr = localtime(&now);

   /* tm_year is the number of years since 1900, so add 1900
    * to get current year as an integer.
    */

   str_year =  g_strdup_printf("%d", ptr->tm_year + 1900);

   /* tm_mon returns a number between 0 and 11 for the month,
    * so adding 1 would make it more understandable; i.e.,
    * 1 = January instead of 0, etc.  However, since the desired
    * data is for the last completed calendar month, in effect,
    * we let the tm_mon call return the current month as a
    * zero-based array, which translates to the current
    * month minus one.
    */

   str_month = g_strdup_printf("%d", ptr->tm_mon);

   /* What if it is January?  In that case, the month/year information
    * needs to reflect the last month of the previous year.
    */

   if (g_strcasecmp(str_month, "0") == 0)
      {
         /* For clarity, use the same function call as before,
          * subtracting one "year".
          */

         str_year =  g_strdup_printf("%d", ptr->tm_year + 1900 - 1);
         str_month = "12";
      }

   g_print("Year and month for query are %s and %s\n", str_year, str_month);

   sql = g_strconcat("select * from tbl_sales_by_division ",
                     "where sales_month = ", str_month,
                     "  and sales_year  = ", str_year, 0L);

   g_print("sql is %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1,"mysql_query problem");
        g_print("mysql_query problem in load_data.\n");
        mysql_close(conx);
        return FALSE;
      }
   /* else continue... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_piechart,
                      "statusbar1")), 1, "mysql_query ok");

   result_set = mysql_store_result(conx);

   /* At this point, result_set holds a list of the reports that
    * are available to be viewed.  Create a GList structure to
    * hold that list.
    */

   while (( row = mysql_fetch_row (result_set)) != 0)
     {
        static gint i = 0;

        division_names[i] = row[2];
        division_sales[i] = atoi(row[3]);

        //g_print("data is %s and %d\n", division_names[i], division_sales[i]);

        total_sales += division_sales[i];
        //g_print("Total sales is %d.\n", total_sales);

        i++;

        //g_print("Number of divisions is %d\n", i);
        number_of_divisions = i;
     }

   mysql_close(conx);

   return TRUE;

}
