#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "support.h"

GtkWidget *window1;
GtkWidget *window2;

void button1_clicked()
{
   GtkWidget *txt;
   gchar *txt1;

   g_print("button1_clicked...\n");

   /* Retrieve the text in the entry widget of the form
    * that holds the button that was clicked.
    */

   txt = lookup_widget(window1, "entry1");
   txt1 = gtk_editable_get_chars(GTK_EDITABLE(txt), 0, -1);

   g_print("%s\n", txt1);

   /* Push the retreived text from above to the label widget of
    * the other window.
    */

   gtk_label_set_text(GTK_LABEL(lookup_widget(window2, "label2")), txt1);
}

void button2_clicked()
{
   GtkWidget *txt;
   gchar *txt2;

   /* button2_clicked has the same structure as button1_clicked,
    * the target widgets are different, however.
    */

   g_print("button2_clicked...\n");

   txt = lookup_widget(window2, "entry2");
   txt2 = gtk_editable_get_chars(GTK_EDITABLE(txt), 0, -1);

   g_print("%s\n", txt2);

   gtk_label_set_text(GTK_LABEL(lookup_widget(window1, "label1")), txt2);
}

void quit_application()
{
   /* The glade XML file identfies this function with the delete-event
    * for both window1 and window2.
    */

   gtk_main_quit();
}

int main (int argc, char *argv[])
{

  GladeXML  *xml;

  gtk_init (&argc, &argv);
  glade_init();

  /* window1 */

  xml = glade_xml_new("cross-window.glade", "window1");

  if (!xml)
     {
       g_print("unable to load interface window1...\n");
       return 1;
     }

  window1 = glade_xml_get_widget(xml, "window1");

  /* Link the called widgets to the window widget such that
   * the called widgets can be retrieved with a lookup_widget()
   * call.
   */

  gtk_object_set_data_full(GTK_OBJECT(window1), "entry1",
                           glade_xml_get_widget(xml, "entry1"), 0L);
  gtk_object_set_data_full(GTK_OBJECT(window1), "label1",
                           glade_xml_get_widget(xml, "label1"), 0L);
  /* Connect the signals to their handlers, as defined in the
   * .glade file.
   */

  glade_xml_signal_autoconnect(xml);

  /* window2 - the code structure is the same as above, only
   * the widgets in question are different.
   */

  xml = glade_xml_new("cross-window.glade", "window2");

  if (!xml)
     {
       g_print("unable to load interface window2...\n");
       return 1;
     }

  window2 = glade_xml_get_widget(xml, "window2");
  gtk_object_set_data_full(GTK_OBJECT(window2), "entry2",
                           glade_xml_get_widget(xml, "entry2"), 0L);
  gtk_object_set_data_full(GTK_OBJECT(window2), "label2",
                           glade_xml_get_widget(xml, "label2"), 0L);
  glade_xml_signal_autoconnect(xml);

  /* Lastly, show the two window widgets. */

  gtk_widget_show (window1);
  gtk_widget_show (window2);

  gtk_main ();
  return 0;
}

